/*
 * 	Driver for VSC452 Timer and Watchdog Timer 
 * 	There are 0- 8 Timers 
 * 	Timer 4  and 8 can be used as Timer/Watchdog Timer 
 *
 */
 
#include <linux/interrupt.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/types.h>
#include <linux/miscdevice.h>
#include <linux/watchdog.h>
#include <linux/fs.h>
#include <linux/ioport.h>
#include <linux/notifier.h>
#include <linux/reboot.h> 
#include <linux/init.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/system.h>
#include <linux/delay.h>
#include <linux/time.h>

#include "vsc452_wdt.h"

int  wdt_irq=5;

#define WDT_VSC_TIMER_0 		( 0 )
#define WDT_VSC_TIMER_1 		( 1 )
#define WDT_VSC_TIMER_2 		( 2 )
#define WDT_VSC_TIMER_3 		( 3 )
#define WDT_VSC_TIMER_4 		( 4 )
#define WDT_VSC_TIMER_5 		( 5 )
#define WDT_VSC_TIMER_6 		( 6 )
#define WDT_VSC_TIMER_7 		( 7 )
#define WDT_VSC_TIMER_8 		( 8 )

int wdt_timer[ 9 ] =
{
   	WDT_VSC_TIMER_0, 
	WDT_VSC_TIMER_1, 
	WDT_VSC_TIMER_2, 
	WDT_VSC_TIMER_3, 
	WDT_VSC_TIMER_4, 
	WDT_VSC_TIMER_5, 
	WDT_VSC_TIMER_6,
	WDT_VSC_TIMER_7, 
	WDT_VSC_TIMER_8, 
};

/* ------------------------------------------------------------------------- 
 * Function Prototypes 
 * ------------------------------------------------------------------------- */

static int vsc_wdt_release(struct inode *inode, struct file *file);
static int vsc_wdt_open(struct inode *inode, struct file *file);
static ssize_t vsc_wdt_write(struct file *file, const char *data, size_t len, loff_t *ppos);
static int vsc_wdt_ioctl(struct inode *inode, struct file *file,unsigned int cmd, unsigned long arg);
static int vsc_notify_sys(struct notifier_block *this, unsigned long code, void *unused);
static irqreturn_t vsc_wdt_interrupt(int irq, void *dev_id);

static int vsc_wdt_stop (int Mode,int TmrNum );
static int vsc_wdt_ping(int WdtNum);
static int vsc_wdt_start( int Mode,int TmrNum );
static int vsc_set_config(TmrWdtConfig_T * tmr_wdt_config);
static int vsc_get_config(TmrWdtConfig_T * tmr_wdt_config);

static struct irqaction wdt_action = {
	.handler	= vsc_wdt_interrupt,
	.flags		= IRQF_SHARED | IRQF_PERCPU | IRQF_DISABLED, 
	.name		= "watchdog",
};

uint32_t TIMERBaseOffset[MAX_TIMER_CTRS] = {
												TIMER_0,
												TIMER_1,
												TIMER_2,
												TIMER_3, 
												WDT_TIMER_4,
												TIMER_5, 
												TIMER_6,
												TIMER_7,
												WDT_TIMER_8												 	   
											};		


/* ------------------------------------------------------------------------- 
 *  Static variables 
 * ------------------------------------------------------------------------- */
 
static void *wdt_remap_addr ;
static struct file_operations vsc_wdt_fops;
static char expect_close;
static TmrWdtConfig_T   wdtAppConf,wdtkernelConf;

int KernelWdt_InUse =0;
int AppWdt_InUse =0;

static struct file_operations vsc_wdt_fops = {
	.owner		= THIS_MODULE,
	.llseek		= no_llseek,
	.write		= vsc_wdt_write,
	.ioctl			= vsc_wdt_ioctl,
	.open		= vsc_wdt_open,
	.release		= vsc_wdt_release,
};

static int exec_cmd(void *arg)
{
	//printk("Kernel thread for PING \n");
	while (1)
	{
		vsc_wdt_ping(KERNEL_WDT);
		msleep(1000);
	}
      return 0;
}



/*Kerenl thread to keep the */
int Kthread(void)
{
	pid_t pid;
	//printk("Kernel thread for KeepAlive \n");
	vsc_wdt_start(WDT_MODE ,0x03);

	pid = kernel_thread(exec_cmd,NULL, 0);
	if (pid < 0) 
	{
		printk(KERN_ERR "Fork Failed, Errno %d\n",-pid);
		return pid;
	}
	//printk("**INIT FINISHED**\n");
	
	return 0;
}

/*
 *	Programming support
 */

 /* iowrite */
inline 
void wdt_write_reg(unsigned long addr, unsigned long val)
{
	*((volatile uint32_t*)addr) = val;
}

/* ioread */
inline
unsigned long wdt_read_reg(unsigned long addr )
{
	return ((*(volatile uint32_t*)addr));
}


/**
 *	vsc_wdt_interrupt:
 */

static irqreturn_t vsc_wdt_interrupt(int irq, void *dev_id)
{
	volatile UINT32 ior ,kcs_status;
	ior = wdt_read_reg ((UINT32) INT_OBSERVE_REG);
	
#if 0
	/* Check if the interrupt is generated by Timer0 */
	if (ior & (UINT32) (TIS0))
	{
		//printk(" vsc_wdt_interupt handler :: TIMER 0 \n");
	}

	/* Check if the interrupt is generated by Timer1 */
	else if (ior & (UINT32)(TIS1))
	{
		//printk(" vsc_wdt_interupt handler :: TIMER 1 \n");
	}
	
	/* Check if the interrupt is generated by Timer2 */
	else if (ior & (UINT32)(TIS2))
	{
		//printk(" vsc_wdt_interupt handler :: TIMER 2 \n");
	}
#endif 

	/*  Kernel Watchdog timer */ 

	/* Check if the interrupt is generated by Timer3 */
	if (ior & (UINT32)(TIS3))
	{
		//printk(" vsc_wdt_interupt handler :: TIMER 3  \n");

		/*Disable the Timer 3 and its interrupt */
		wdt_write_reg((uint32_t) (TIMERBaseOffset[3]+TMCX), (uint32_t)0x00);
		wdt_write_reg((uint32_t) (TIMERBaseOffset[3]+TICX), (uint32_t)0x00);	   
		wdt_write_reg((uint32_t) (TIMERBaseOffset[3]+TISX), (uint32_t)0x00);


		/*Set KCS register (0xB0200E00) at 0x09*/

		// Clear IBF and OBF  Flags  
		wdt_write_reg((uint32_t) KCS2_STATUS, (uint32_t)0x00);

		// Enable LPC Interrupt 
		// wdt_write_reg((uint32_t) KCS2_MODE, (uint32_t)0x00);

		//KCS register (0xB0200E00) at 0x09 
		wdt_write_reg((uint32_t) KCS2_DATA, (uint32_t)0x09);
		//Setting OBF  Flags  
		ior = wdt_read_reg ((UINT32) INT_OBSERVE_REG);
		kcs_status = (uint32_t) (kcs_status | (uint32_t) 0x01);
		wdt_write_reg((uint32_t) KCS2_STATUS, (uint32_t)kcs_status);		 	


		/*  POS Regsiter must be set to indicate the Kernel Watchdog expiration  (0xd4) 
                   check for 0xf6 and if it is kernel panic update the different code   */
               if ( 0xF6 == wdt_read_reg ((uint32_t)POS_REG))
               {
			wdt_write_reg((uint32_t) POS_REG, (uint32_t)0xda);
               }
		else 
               {     
			wdt_write_reg((uint32_t) POS_REG, (uint32_t)0xd4);
               }

		/* SRAM must be set to 0x52 to indicate OEM_RESET_PANIC condition */
		wdt_write_reg((uint32_t) SRAM_REG, (uint32_t)0x52);

		/*Zeroing the Timer 4 counter */
		wdt_write_reg((uint32_t) (TIMERBaseOffset[4]+TMCX), (uint32_t)0x00);
		wdt_write_reg((uint32_t) (TIMERBaseOffset[4]+TISX), (uint32_t)0x00);	
		wdt_write_reg((uint32_t) (TIMERBaseOffset[4]+TICX), (uint32_t)0x10);	


		//Reseting the SP 
		wdt_write_reg((uint32_t) (TIMERBaseOffset[4]+TISX), (uint32_t)0x01);
		wdt_write_reg((uint32_t) (TIMERBaseOffset[4]+TMCX), (uint32_t)0x03);
 
		return IRQ_HANDLED;
	}

	/* Check if the interrupt is generated by Timer4 or kernel Watchdog Timer */
	else if (ior & (UINT32)(TIS4))
	{
		//printk(" vsc_wdt_interupt handler :: TIMER 4 :KERNEL WDT RESET \n");
#if 0
		wdt_write_reg((uint32_t) (TIMERBaseOffset[4]+TICX), (uint32_t)0x00);	
		/*Reseting the SP */
		wdt_write_reg((uint32_t) (TIMERBaseOffset[4]+TISX), (uint32_t)0x01);
		wdt_write_reg((uint32_t) (TIMERBaseOffset[4]+TMCX), (uint32_t)0x03);


#endif 	
		return IRQ_HANDLED;
	}
	
#if 0
	/* Check if the interrupt is generated by Timer5 */
	else if (ior & (UINT32)(TIS5))
	{
	//printk(" vsc_wdt_interupt handler :: TIMER 5  \n");
	}

	/* Check if the interrupt is generated by Timer6 */
	else if (ior & (UINT32)(TIS6))
	{
	//printk(" vsc_wdt_interupt handler :: TIMER 6  \n");
	}
#endif 

	/*  Application  Watchdog timer */  
	/* Check if the interrupt is generated by Timer7 */
	else if (ior & (UINT32)(TIS7))
	{
		//printk(" vsc_wdt_interupt handler :: TIMER 7  \n");

		/*Disable the Timer 7 and its interrupt */
		wdt_write_reg((uint32_t) (TIMERBaseOffset[7]+TMCX), (uint32_t)0x00);
		wdt_write_reg((uint32_t) (TIMERBaseOffset[7]+TICX), (uint32_t)0x00);	   
		wdt_write_reg((uint32_t) (TIMERBaseOffset[7]+TISX), (uint32_t)0x00);
		
		
                 /*Set KCS register (0xB0200E00) at 0x09*/
			
		// Clear IBF and OBF  Flags  
		wdt_write_reg((uint32_t) KCS2_STATUS, (uint32_t)0x00);
		
		// Enable LPC Interrupt 
		// wdt_write_reg((uint32_t) KCS2_MODE, (uint32_t)INTR_EN_MASK);

		//KCS register (0xB0200E00) at 0x09 
		wdt_write_reg((uint32_t) KCS2_DATA, (uint32_t)0x09);

                 //Setting OBF  Flags  
                 ior = wdt_read_reg ((UINT32) INT_OBSERVE_REG);
                 kcs_status = (uint32_t) (kcs_status | (uint32_t) 0x01);
		 wdt_write_reg((uint32_t) KCS2_STATUS, (uint32_t)kcs_status);	

			
		 /*  POS Regsiter must be set to indicate the AppliactionWatchdog expiration  (0xd4)*/
		wdt_write_reg((uint32_t) POS_REG, (uint32_t)0xd8);

		/* SRAM must be set to 0x52 to indicate OEM_RESET_PANIC condition */
		wdt_write_reg((uint32_t) SRAM_REG, (uint32_t)0x52);

		/*Zeroing the Timer 4 counter */
		wdt_write_reg((uint32_t) (TIMERBaseOffset[8]+TMCX), (uint32_t)0x00);
		wdt_write_reg((uint32_t) (TIMERBaseOffset[8]+TISX), (uint32_t)0x00);	
		wdt_write_reg((uint32_t) (TIMERBaseOffset[8]+TICX), (uint32_t)0x10);	

		//Reseting the SP 
		wdt_write_reg((uint32_t) (TIMERBaseOffset[8]+TISX), (uint32_t)0x01);
		wdt_write_reg((uint32_t) (TIMERBaseOffset[8]+TMCX), (uint32_t)0x03);

	
		return IRQ_HANDLED;    
		
	}

	/* Check if the interrupt is generated by Timer8 and Application Watchdog timer */
	else if(ior & (UINT32)(TIS8))
	{
		//printk(" vsc_wdt_interupt handler :: TIMER 8 : APPLICATION WDT  RESET \n");

	#if 0
		/*Zeroing the Timer 8 counter */
		wdt_write_reg((uint32_t) (TIMERBaseOffset[8]+TICX), (uint32_t)0x00);	
		

		/*Reseting the SP */
		wdt_write_reg((uint32_t) (TIMERBaseOffset[8]+TISX), (uint32_t)0x01);
		wdt_write_reg((uint32_t) (TIMERBaseOffset[8]+TMCX), (uint32_t)0x03);
		
	#endif 	
		return IRQ_HANDLED;
	}

	else {
 
		return IRQ_NONE;
	}

	
}



/**
 *	vsc_wdt_start:
 *
 *	Start the watchdog driver.
 */

static int vsc_wdt_start(int Mode ,int TmrNum )
{
      
#if 0	
     if (Mode==TMR_MODE)
     	{
     	      //printk("  vsc_wdt_start = TMR_MODE   \n");
    	     
		if ( TmrNum==0 ||TmrNum==1 ||TmrNum==2 ||TmrNum== 5 ||TmrNum==6)
		{
			/*Load the TimeInterval in TICX register */////////////real time vaue 
			wdt_write_reg((uint32_t) (TIMERBaseOffset[TmrNum]+TICX), (uint32_t)0x0ffffff);

			/* Enable Timer Interrupt in   TISX register  */
			wdt_write_reg((uint32_t) (TIMERBaseOffset[TmrNum]+TISX), (uint32_t)0x01);

			/*Enable the Timer to start in TMCX register */
			wdt_write_reg((uint32_t) (TIMERBaseOffset[TmrNum]+TMCX), (uint32_t)0x01);
		}

		/*check if these timers are already used for Watchdog Timer */	 
              if ( TmrNum==3 ||TmrNum==4 ||TmrNum==7 ||TmrNum== 8) 
    	     	{

		 /*check if both kernel an App Watchdog Timer is set */	
		  if (KernelWdt_InUse ==1 && AppWdt_InUse ==1)
    	     	  	{
    	     	  	  	 return -1;
    	     	  	}
		  
		  /*check if  kernel Watchdog Timer is set and trying to use timer 3 and 4  */	
		  if (KernelWdt_InUse ==1 && (TmrNum==3 ||TmrNum==4) )
    	     	  	{
    	     	  	   	 return -1;
    	     	  	}	
		  
		  /*check if App Watchdog Timer is set and trying to use  timer 7 and 8 */		
		    if (AppWdt_InUse  ==1 && (TmrNum==7 ||TmrNum== 8 ))
    	     	  	{
    	     	  	   	 return -1;
    	     	  	}
			else
			{
				 /*Load the TimeInterval in TICX register */////////////real time vaue 
		  		 wdt_write_reg((uint32_t) (TIMERBaseOffset[TmrNum]+TICX), (uint32_t)0x0ffffff);

		  		 /* Enable Timer Interrupt in   TISX register  */
				 wdt_write_reg((uint32_t) (TIMERBaseOffset[TmrNum]+TISX), (uint32_t)0x01);

		 		  /*Enable the Timer to start in TMCX register */
    	     	   		 wdt_write_reg((uint32_t) (TIMERBaseOffset[TmrNum]+TMCX), (uint32_t)0x01);
						
			}
	     	}
	}

#endif 

	if (Mode==WDT_MODE)
	{
		
		/* check if the Kerenel and  App is configured already  */
		
		if (KernelWdt_InUse ==1  && AppWdt_InUse ==1) 
		{
			//printk("  check if the Kerenel and  App is configured already in use  \n");
			return -1;
		}
		/*check if  kernel Watchdog Timer is set and trying to use timer 3 and 4 ,which is alraedy in use */	
		if (KernelWdt_InUse ==1 && (TmrNum==3 ||TmrNum==4 ))
		{
			//printk("  check if  kernel Watchdog Timer is set and trying to use timer 3 and 4 ,which is alraedy in use   \n");
			return -1;
		}	

		/*check if App Watchdog Timer is set and trying to use  timer 7 and 8 ,which is alraedy in use*/		
		if (AppWdt_InUse  ==1 && (TmrNum==7 ||TmrNum== 8 ))
		{
			//printk("  check if App Watchdog Timer is set and trying to use  timer 7 and 8 ,which is alraedy in use \n");
			return -1;
		}

		/* check for kernel Watchdog to start */
		if (TmrNum==3 ||TmrNum==4)             
		{
			//printk("  START :  kernel Watchdog Timer   \n");
			KernelWdt_InUse =1;
			vsc_wdt_ping(KERNEL_WDT);
		}
		
		/* check for App Watchdog to start */	 
		if (TmrNum==7 ||TmrNum==8)           
		{
			//printk("  START :  Application Watchdog Timer  \n");
			AppWdt_InUse =1;
			vsc_wdt_ping(APP_WDT);
		}

	}
	else 
	{
		return -1 ;
	} 
	 return 0;  
   }


/**
 *	vsc_wdt_stop:
 *
 *	Stop the watchdog driver.
 */

static int vsc_wdt_stop (int Mode,int TmrNum  )
{
	
	if (Mode==WDT_MODE)
	{
		if (KernelWdt_InUse ==1 && (TmrNum==3 ||TmrNum==4) )
		{
			KernelWdt_InUse=0;
			wdt_write_reg((uint32_t) (TIMERBaseOffset[3]+TMCX), (uint32_t)0x00);
			wdt_write_reg((uint32_t) (TIMERBaseOffset[3]+TICX), (uint32_t)0x00);	   
			wdt_write_reg((uint32_t) (TIMERBaseOffset[3]+TISX), (uint32_t)0x00);

			wdt_write_reg((uint32_t) (TIMERBaseOffset[4]+TMCX), (uint32_t)0x00);
			wdt_write_reg((uint32_t) (TIMERBaseOffset[4]+TICX), (uint32_t)0x00);	   
			wdt_write_reg((uint32_t) (TIMERBaseOffset[4]+TISX), (uint32_t)0x00);

		}
		/*check if App Watchdog Timer is set and trying to stop  timer 7 and 8 */		
		else if (AppWdt_InUse  ==1 && (TmrNum==7 ||TmrNum== 8 ))
		{
			AppWdt_InUse=0;
			wdt_write_reg((uint32_t) (TIMERBaseOffset[7]+TMCX), (uint32_t)0x00);
			wdt_write_reg((uint32_t) (TIMERBaseOffset[7]+TICX), (uint32_t)0x00);	   
			wdt_write_reg((uint32_t) (TIMERBaseOffset[7]+TISX), (uint32_t)0x00);	  

			wdt_write_reg((uint32_t) (TIMERBaseOffset[8]+TMCX), (uint32_t)0x00);
			wdt_write_reg((uint32_t) (TIMERBaseOffset[8]+TICX), (uint32_t)0x00);	   
			wdt_write_reg((uint32_t) (TIMERBaseOffset[8]+TISX), (uint32_t)0x00);
		}   
		else 
		{
		//	printk(KERN_ERR "  STOP ERROR  :   Watchdog Timer NOT ENABLED TO STOP \n");
			return -1 ;
		} 

	}
#if 0
	if (Mode==TMR_MODE)
	{
		if ( TmrNum==0 ||TmrNum==1 ||TmrNum==2 ||TmrNum== 5 ||TmrNum==6)
		{
			/*Stop the Timer  */
			wdt_write_reg((uint32_t) (TIMERBaseOffset[TmrNum]+TMCX), (uint32_t)0x00);
		}

		/*check if these timers are already used for Watchdog Timer */	 
		if ( TmrNum==3 ||TmrNum==4 ||TmrNum==7 ||TmrNum== 8) 
		{
			/*check if both kernel an App Watchdog Timer is set */	
			if (KernelWdt_InUse ==1 && AppWdt_InUse ==1)
			{
				return -1;
			}
			/*check if  kernel Watchdog Timer is set and trying to stop  timer 3 and 4  */	
			if (KernelWdt_InUse ==1 && (TmrNum==3 ||TmrNum==4) )
			{
				return -1;
			}	
			/*check if App Watchdog Timer is set and trying to stop  timer 7 and 8 */		
			if (AppWdt_InUse  ==1 && (TmrNum==7 ||TmrNum== 8 ))
			{
				return -1;
			}
			else
			{
				wdt_write_reg((uint32_t)(TIMERBaseOffset[TmrNum]+TMCX), (uint32_t)0x00);
			}
		}
	}
#endif 
	 
	 return 0;
}

		
 
/**
 *	vsc_wdt_ping:
 *
 *	Reload counter one with the watchdog . 
 *	
 */

static int vsc_wdt_ping(int WdtType)
	{
		//// check here if they are enabled  KernelWdt_InUse =1; 
		
		if (WdtType==KERNEL_WDT) 				// kernel Watchdog  
		{
			
			if (KernelWdt_InUse ==1)
				{
				//printk("  vsc_wdt_ping  : KERNEL_WDT : PING SUCESS \n");

				wdtkernelConf.PreTimeOutInterval =COUNT_VAL_8_SEC;
				wdtkernelConf.TimeInterval=COUNT_VAL_10_SEC;


				/*Clear the Pretimeout and Timeout Interval for Timer and Watchdog timer */
				wdt_write_reg((uint32_t) (TIMERBaseOffset[3]+TMCX), (uint32_t)0x00);
				wdt_write_reg((uint32_t) (TIMERBaseOffset[3]+TISX), (uint32_t)0x00);	   
				wdt_write_reg((uint32_t) (TIMERBaseOffset[4]+TMCX), (uint32_t)0x00);
				wdt_write_reg((uint32_t) (TIMERBaseOffset[4]+TISX), (uint32_t)0x00);

				/*Load Pretimeout and Timeout Interval for Timer and Watchdog timer values*/
				wdt_write_reg((uint32_t) (TIMERBaseOffset[3]+TICX), (uint32_t)wdtkernelConf.PreTimeOutInterval);
				wdt_write_reg((uint32_t) (TIMERBaseOffset[4]+TICX), (uint32_t)wdtkernelConf.TimeInterval);

				/*Enable the interrupt for  Timer and Watchdog timer values*/
				wdt_write_reg((uint32_t) (TIMERBaseOffset[3]+TISX), (uint32_t)0x01);
				wdt_write_reg((uint32_t) (TIMERBaseOffset[4]+TISX), (uint32_t)0x01);

				/*Start  Timer and Watchdog timer */
				wdt_write_reg((uint32_t) (TIMERBaseOffset[3]+ TMCX), (uint32_t)0x01);
				wdt_write_reg((uint32_t) (TIMERBaseOffset[4]+TMCX), (uint32_t)0x03);	

				
			}
			else 
			{
				printk(KERN_ERR "  ERROR PINGING :  kernel watchdog Timer NOT ENABLED   \n");
				return -1;
			}
		}


		if (WdtType==APP_WDT) 					// Appliaction Watchdog  
		{
			

			if (AppWdt_InUse ==1)
			{
				//printk("  vsc_wdt_ping  : APP_WDT : PING SUCCESS \n");

				wdt_write_reg((uint32_t) (TIMERBaseOffset[7]+TMCX), (uint32_t)0x00);
				wdt_write_reg((uint32_t) (TIMERBaseOffset[7]+TISX), (uint32_t)0x00);	   
				 wdt_write_reg((uint32_t) (TIMERBaseOffset[8]+TMCX), (uint32_t)0x00);
				wdt_write_reg((uint32_t) (TIMERBaseOffset[8]+TISX), (uint32_t)0x00);

				wdt_write_reg((uint32_t) (TIMERBaseOffset[7]+TICX), (uint32_t)wdtAppConf.PreTimeOutInterval);
				wdt_write_reg((uint32_t) (TIMERBaseOffset[8]+TICX), (uint32_t)wdtAppConf.TimeInterval);

				/*Enable the interrupt for  Timer and Watchdog timer values*/
				wdt_write_reg((uint32_t) (TIMERBaseOffset[7]+TISX), (uint32_t)0x01);
				wdt_write_reg((uint32_t) (TIMERBaseOffset[8]+TISX), (uint32_t)0x01);

				wdt_write_reg((uint32_t) (TIMERBaseOffset[7]+TMCX), (uint32_t)0x01);
				wdt_write_reg((uint32_t) (TIMERBaseOffset[8]+TMCX), (uint32_t)0x03);
				
			}
			else
			{
				printk(KERN_ERR "  ERROR PINGING :  Application  watchdog Timer NOT ENABLED  \n");
				return -1;
			}
		}

	return 0;
	}

/**
 *	wdt_open:
 *	@inode: inode of device
 *	@file: file handle to device
 *
 *	The watchdog device has been opened. The watchdog device is single
 *	open and on opening we load the counters. Counter zero is a 100Hz
 *	cascade, into counter 1 which downcounts to reboot. When the counter
 *	triggers counter 2 downcounts the length of the reset pulse which
 *	set set to be as long as possible.
 */

static int vsc_wdt_open(struct inode *inode, struct file *file)
{
	
#if 0
	if(test_and_set_bit(0, &wdt_is_open))
		return -EBUSY;
	/*
	*	Activate
	*/
#endif 
	//vsc_wdt_start();
	return 0;
}

/**
 *	vsc_wdt_release:
 *	@inode: inode to board
 *	@file: file handle to board
 *
 *	The watchdog has a configurable API. There is a religious dispute
 *	between people who want their watchdog to be able to shut down and
 *	those who want to be sure if the watchdog manager dies the machine
 *	reboots. In the former case we disable the counters, in the latter
 *	case you have to open it again very soon.
 */
static int vsc_wdt_release(struct inode *inode, struct file *file)
{
	//int TmrNum =0;
	//int Mode=0;

#if 0
	//printk("  vsc_wdt_release  \n");

	if (expect_close == 42) 
	{
		vsc_wdt_stop(wdtAppConf.Mode,0x07);
	} 
	else 
	{
		printk(KERN_CRIT "wdt: WDT device closed unexpectedly.  WDT will not stop!\n");
		vsc_wdt_ping(wdtAppConf.WdtType);
	}
	expect_close = 0;
#endif 	
	return 0;
}

/**
 *	vsc_wdt_write:
 *	@file: file handle to the watchdog
 *	@buf: buffer to write (unused as data does not matter here
 *	@count: count of bytes
 *	@ppos: pointer to the position to write. No seeks allowed
 *
 *	A write to a watchdog device is defined as a keepalive signal. Any
 *	write of data will do, as we we don't define content meaning.
 */

static ssize_t vsc_wdt_write (struct file *file, const char __user *buf, size_t count, loff_t *ppos)
{
	

	if(count) {
		//if (!nowayout) {
		size_t i;

		/* In case it was set long ago */
		expect_close = 0;

		for (i = 0; i != count; i++) 
		{
			char c;
			if (get_user(c, buf + i))
			return -EFAULT;
			if (c == 'V')
			expect_close = 42;
		}
		//}
		//vsc_wdt_ping( WdtNum);
	}
	return count;
}

static int vsc_set_config(TmrWdtConfig_T *tmr_wdt_config)
{

	TmrWdtConfig_T * tmrwdtCon;
	tmrwdtCon=tmr_wdt_config;

	wdtAppConf.Mode= tmrwdtCon->Mode;
	wdtAppConf.WdtType= tmrwdtCon->WdtType;
	wdtAppConf.TmrNum= tmrwdtCon->TmrNum;
	wdtAppConf.PreTimeOutInterval= tmrwdtCon->PreTimeOutInterval;
	wdtAppConf.TimeInterval= tmrwdtCon->TimeInterval;
	wdtAppConf.TimerUse= tmrwdtCon->TimerUse;
	wdtAppConf.TmrAction= tmrwdtCon->TmrAction;

	return 0;
}


static int vsc_get_config(TmrWdtConfig_T *tmr_wdt_config)
{

	TmrWdtConfig_T * tmrwdtCon;
	tmrwdtCon=tmr_wdt_config;

	tmrwdtCon->Mode= wdtAppConf.Mode;
	tmrwdtCon->WdtType=  wdtAppConf.WdtType;
	tmrwdtCon->TmrNum= wdtAppConf.TmrNum;
	tmrwdtCon->PreTimeOutInterval= wdtAppConf.PreTimeOutInterval ;
	tmrwdtCon->TimeInterval=wdtAppConf.TimeInterval ;
	tmrwdtCon->TimerUse= wdtAppConf.TimerUse;
	tmrwdtCon->TmrAction=wdtAppConf.TmrAction ;

	return 0;

}

static int vsc_wdt_ioctl(struct inode *inode, struct file *file,unsigned int cmd, unsigned long arg)
{

	TmrWdtConfig_T  tmr_wdt_conf;

	/*Getting data from user space to driver space  */
	if ( copy_from_user(&tmr_wdt_conf, (void*)arg,sizeof( TmrWdtConfig_T)) )
	{ 
		//printk(" Error  getting the Data from user space (TmrWdtConfig_T) in ioctl   \n" );
		return -EFAULT;
	}
	switch (cmd)
	{
			
		case WDT_KEEPALIVE:
			vsc_wdt_ping( tmr_wdt_conf.WdtType );
		break;
		
		case WDT_TMR_ENABLE:
#if 0
			if (tmr_wdt_conf.Mode==TMR_MODE)
			{

			//printk("  IOCTL  WDT_TMR_ENABLE **  TMR_MODE \n");
			vsc_wdt_start(tmr_wdt_conf.Mode ,tmr_wdt_conf.TmrNum);   
			}
#endif 
			if (tmr_wdt_conf.Mode==WDT_MODE)
			{
				if (tmr_wdt_conf.WdtType==KERNEL_WDT)
				{
					vsc_wdt_start(tmr_wdt_conf.Mode ,0x03);
				}

				if (tmr_wdt_conf.WdtType==APP_WDT)
				{
					vsc_wdt_start(tmr_wdt_conf.Mode ,0x07);
				}
			}
			break;
			
		case WDT_TMR_DISABLE:
			 #if 0
			 if (tmr_wdt_conf.Mode==TMR_MODE)
			 	{
				    //printk("  IOCTL  WDT_TMR_DISABLE **  TMR_MODE \n");
			 	     vsc_wdt_stop(tmr_wdt_conf.Mode ,tmr_wdt_conf.TmrNum);   
			 	}
			 #endif 
			if (tmr_wdt_conf.Mode==WDT_MODE)
			{
				if (tmr_wdt_conf.WdtType==KERNEL_WDT	)
				{
					vsc_wdt_stop(tmr_wdt_conf.Mode ,0x03);
				}

				if (tmr_wdt_conf.WdtType==APP_WDT)
				{
					vsc_wdt_stop(tmr_wdt_conf.Mode ,0x07);
				}
			}
			break;
			
		case WDT_TMR_SETCONFIG:
			vsc_set_config(&tmr_wdt_conf);
			break;
			
		case WDT_TMR_GETCONFIG:
			vsc_get_config(&tmr_wdt_conf);
			if (copy_to_user((void*)arg, &tmr_wdt_conf , sizeof(TmrWdtConfig_T))) 
			{ 
				//printk(" Error sending the App Wdt Configuration data to  User Data  \n" );
				return -EFAULT;
			}
			 break;
			 
	 	case WDT_TMR_GET_CUR_CNT:
			break;

		default:
			return -ENOIOCTLCMD;
			//vsc_get_cur_cnt();	
	}

	return 0;
}



static struct miscdevice wdt_kernl_miscdev =
{
	.minor		= KERNL_WDT_MINOR,
	.name		= "watchdog0",
	.fops			= &vsc_wdt_fops,
};

static struct miscdevice wdt_app_miscdev =
{
	.minor		= APP_WDT_MINOR,
	.name		= "watchdog1",
	.fops			= &vsc_wdt_fops,
};

static struct notifier_block vsc_wdt_notifier =
{
	.notifier_call = vsc_notify_sys,
};

static int vsc_notify_sys(struct notifier_block *this, unsigned long code, void *unused)
{
	if (code == SYS_DOWN || code == SYS_HALT)

	//////////modify here for both kernel and application watchdog timer 
	//printk("Inside  vsc_notify_sys  \n");
	vsc_wdt_stop (WDT_MODE,0x03);		/* Turn the WDT off */
	vsc_wdt_stop (WDT_MODE,0x07);
	return NOTIFY_DONE;
}

static int __init wdt_vsc_init(void)
{
	int err,ret=0;
	uint32_t wdt_base;

	//printk(" wdt_vsc_init \n");

	/* Remap registers */
	 wdt_base= WDT_BASE_OFFSET;
	if ((wdt_remap_addr = ioremap_nocache(wdt_base, WDT_REG_SIZE)) == NULL) 
	{
		printk(KERN_ERR "Failed to map WDT space %08x-%08x to memory",wdt_base, wdt_base + WDT_REG_SIZE - 1);
		ret = -EIO;
		return ret;
	}

	/* check gpio memory region */
	if ((err = check_mem_region(wdt_base, WDT_REG_SIZE)) < 0)  
	{
		printk(KERN_ERR "WDT  IO space %08x-%08x already in use (err %d)",wdt_base, wdt_base + WDT_REG_SIZE - 1, err);
		ret = -EBUSY;
		iounmap( wdt_remap_addr); 
		return ret;
	}

	/* request for the memory region */	
	if (request_mem_region(wdt_base,WDT_REG_SIZE,DRIVER_NAME)  == NULL) 
	{ 
		printk(KERN_ERR "IO space %08x-%08x adc request mem region failed (err %d)",wdt_base, wdt_base + WDT_REG_SIZE - 1, err);
		ret = -ENODEV;
		iounmap( wdt_remap_addr); 
		return ret;
	}

	setup_irq(5, &wdt_action);

	ret = register_reboot_notifier(&vsc_wdt_notifier);
	if(ret) 
	{
		printk(KERN_ERR "vsc452_wdt_notifier : cannot register reboot notifier (err=%d)\n", ret);
		ret= -ENODEV;
		free_irq(wdt_irq, NULL); //////
		release_mem_region(wdt_base,WDT_REG_SIZE);
		iounmap( wdt_remap_addr); 
		return ret;
	}
	
	ret = misc_register(&wdt_kernl_miscdev );
	if (ret) 
	{
		printk(KERN_ERR "wdt: cannot register miscdev on minor=%d (err=%d)\n",KERNL_WDT_MINOR, ret);
		 unregister_reboot_notifier(&vsc_wdt_notifier);
		free_irq(wdt_irq, NULL); //check if necessary for setup irq 
		release_mem_region(wdt_base,WDT_REG_SIZE);
		iounmap( wdt_remap_addr); 
		return ret;
	}

       ret = misc_register(&wdt_app_miscdev );
	if (ret) 
	{
		printk(KERN_ERR "wdt: cannot register miscdev on minor=%d (err=%d)\n",APP_WDT_MINOR, ret);
		 unregister_reboot_notifier(&vsc_wdt_notifier);
		free_irq(wdt_irq, NULL);  //
		release_mem_region(wdt_base,WDT_REG_SIZE);
		iounmap( wdt_remap_addr); 
		misc_deregister(&wdt_kernl_miscdev );
		return ret;
	}
	
	Kthread();
  	printk(KERN_ALERT "Watchdog Module  was  sucessully loaded \n");
 
       /*Initialising the POS register to 0x00 */      
	//wdt_write_reg((uint32_t) POS_REG, (uint32_t)0x00);

	return 0;
}

 static void __exit wdt_vsc_exit(void)
{
       	uint32_t wdt_base ;
	wdt_base= WDT_BASE_OFFSET;
	
	misc_deregister(&wdt_kernl_miscdev );
	misc_deregister(&wdt_app_miscdev );
	
        unregister_reboot_notifier(&vsc_wdt_notifier);
 	free_irq(wdt_irq, NULL); //
 	
	/* release the memory occupied */  
	release_mem_region(wdt_base,WDT_REG_SIZE);

	/* Unmap registers */
	iounmap( wdt_remap_addr); 

	printk(KERN_ALERT "Watchdog Module  was  sucessully Unloaded \n");
}

module_init(wdt_vsc_init);
module_exit(wdt_vsc_exit);

MODULE_AUTHOR("American Megatrends Inc.");
MODULE_DESCRIPTION("VSC452  Watchdog Driver )");
MODULE_LICENSE("GPL");

