/*
 * Copyright (C) 2007 American Megatrends Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifdef __UBOOT__	
#include <common.h>
#endif
#include "spiflash.h"
#ifdef	CFG_FLASH_SPI_DRIVER

/* SST specific SPI Commands or OP Codes */
#define OPCODE_AAI     		0xAD
#define OPCODE_BYTEWRITE	0x02	
/* Program data count*/
#define PROGRAM_DATA_COUNT  	0x02


/* prototype from generic routines */
extern int spi_generic_wait_till_ready(int bank, struct spi_ctrl_driver *ctrl_drv);
extern int spi_generic_write_disable(int bank, struct spi_ctrl_driver *ctrl_drv);

/* Name, ID1, ID2 , Size, Erase regions, { Offset, Erase Size, Erase Block Count } */
static struct spi_flash_info sst_data [] = 
{
	/* SST 64K Sectors */
	{ "SST 25VF016B" , 0xBF, 0x2541, 0x200000, 1, {{ 0, 64 * 1024, 32  },} },

};


static
int 
sst_probe(int bank,struct spi_ctrl_driver *ctrl_drv, struct spi_flash_info *chip_info)
{
	int retval;

	retval =  spi_generic_probe(bank, ctrl_drv,chip_info,"sst",sst_data,ARRAY_SIZE(sst_data));
	if (retval == -1)
		return retval;

	/* Unprotect all sectors */
	if (spi_generic_write_status(bank,ctrl_drv,0x0) < 0)
		printk("sst: Unable to Unprotect all sectors\n");


	return retval;
}


int
sst_write(struct map_info *map, loff_t addr, size_t bytes, const unsigned char *buff)
{
	struct spi_flash_private * priv = map->fldrv_priv;
	int bank = map->map_priv_1;
	struct spi_ctrl_driver *ctrl_drv = priv->ctrl_drv;
	int retval;
	unsigned char command[4];
	size_t transfer;
	
	/* flag to indicate we need to give address for the first time */
	int write_addr =1;
	int cmd_len =0;	
	
	
	/*validate bytes */
	if (0==bytes)
	{	
		printk ( "zero lenght data  sent\n");
		return 0;
	}
	
	if ( (bytes % 2 ) )
		printk("we got odd bytes\n"); 

	down(&priv->chip_drv->lock);
	
	/*wait here till spi flash is ready to write*/
	if (spi_generic_wait_till_ready(bank,ctrl_drv))
	{
		up(&priv->chip_drv->lock);
		return -1;
	}
	
	/* Send write enable */
	spi_generic_write_enable(bank,ctrl_drv);

	while(bytes)
	{
		/* Wait until finished previous command. */
                if (spi_generic_wait_till_ready(bank,ctrl_drv))
                {
                        up(&priv->chip_drv->lock);
                        return -1;
                }

                transfer = PROGRAM_DATA_COUNT;

                /* Set up command buffer. */
                command[0] = OPCODE_AAI;
		cmd_len = 1;
		if (write_addr )
		{
	                command[1] = addr >> 16;
        	        command[2] = addr >> 8;
                	command[3] = addr;
			cmd_len = 4;
			write_addr = 0;
		}	
		/*Add a Dummy byte if count is odd */
		if ( bytes < transfer )
		{
			spi_generic_write_disable(bank, ctrl_drv);
			if ( spi_generic_wait_till_ready(bank,ctrl_drv))
			{
				up(&priv->chip_drv->lock);
				return -1;
			}
			
			printk ( "writing odd byte\n"); 		
			command[0] = OPCODE_BYTEWRITE;
			command[1] = addr >> 16;
			command[2] = addr >> 8;
			command[3] = addr;
			cmd_len = 4;
			transfer = 1;
			/*Writing going to end */
	                retval = ctrl_drv->spi_transfer(bank,command,cmd_len ,SPI_WRITE,
                                                (unsigned char *)buff, transfer);

			if ( retval < 0 )
			{
				up(&priv->chip_drv->lock);
				return -1;
			}
			if ( spi_generic_wait_till_ready(bank,ctrl_drv) )
			{
				up(&priv->chip_drv->lock);
				return -1;	
			}
			/* writing finished */
			up (&priv->chip_drv->lock);
			bytes--;
			continue;	
		}
		else
		{		
	                /* Issue Controller Transfer Routine */
        	        retval = ctrl_drv->spi_transfer(bank,command,cmd_len ,SPI_WRITE,
                                                (unsigned char *)buff, transfer);
		}
                if (retval < 0)
                {
                        up(&priv->chip_drv->lock);
                        return retval;
                }
                addr+=2;
                buff+=2;
                bytes-=2;

	}

	if ( spi_generic_wait_till_ready(bank,ctrl_drv))
	{
		up(&priv->chip_drv->lock);
		return -1;
	}
	/* Send Write Disable command to stop AAI*/
	spi_generic_write_disable(bank,ctrl_drv );
	
	if ( spi_generic_wait_till_ready(bank,ctrl_drv))
	{
		up(&priv->chip_drv->lock);
		return -1;
	}
	up(&priv->chip_drv->lock);
	return 0;
}

struct spi_chip_driver sst_driver =
{
	.name 		= "sst",
	.module 	= THIS_MODULE,
	.probe	 	= sst_probe,
	.erase_sector 	= spi_generic_erase,
	.read_bytes	= spi_generic_read,
	.write_bytes	= sst_write,
};



int 
sst_init(void)
{
	init_MUTEX(&sst_driver.lock);
#ifdef __UBOOT__	/* MIPS */
	sst_driver.probe	 	= sst_probe;
	sst_driver.erase_sector 	= spi_generic_erase;
	sst_driver.read_bytes		= spi_generic_read;
	sst_driver.write_bytes		= sst_write;
#endif
	register_spi_chip_driver(&sst_driver);
	return 0;
}


void 
sst_exit(void)
{
	init_MUTEX(&sst_driver.lock);
	unregister_spi_chip_driver(&sst_driver);
	return;
}


module_init(sst_init);
module_exit(sst_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("American Megatrends Inc");
MODULE_DESCRIPTION("MTD SPI driver for SST flash chips");

#endif
